#
# Multidimensional Polytomous IRT for RR Data
# Fox and Klein Entink 2012
#
#

MIRTRR <- function(Y,X,Q,XG,RR){

#initialize

N <- nrow(Y)
K <- ncol(Y)
C <- max(Y[!is.na(Y)])
if(mean(X) != 1){
	P <- ncol(X) ## number of covariates. X[,1] = 1s for the intercept
}
if(mean(X)==1){P <- 1} # no covariates, only 1s for the intercept.

beta  <- t(matrix(c(-1000,seq(-1.5,1.5,3/(C-2)),1000),nrow=C+1,ncol=K)) #thresholds
##RR 	<- matrix(1,ncol=K,nrow=N) # all data are RR
D 	<- matrix(1,ncol=1,nrow=N*K) # no missings
D[which(is.na(Y))] <- 0 #identify missings
D <- matrix(D,nrow=N,ncol=K)

#RR characteristics
#p1 = .666 #2/3 true response
#p2 = .25  #forced category response probability

#CAPS and AEQ Data 
p1 = .60  ## 60% true response
p2 = .20  ## 20% forced category response probability

IC <- FALSE #No intercept

## EAP estimates
burnin <- 1000

## MCMC Storage
	Mutheta <- matrix(0,ncol=Q,nrow=XG)
	EAPtheta <- matrix(0,ncol=Q,nrow=N)
	MA <- matrix(0,ncol=K*Q,XG)
	MSigmaP <- matrix(NA,ncol=Q*Q,nrow=XG)
	Mbeta <- matrix(0,ncol=K*(C-1),nrow=XG)
  	MBReg <- array(0,dim=c(XG,P,Q))
	Mloglik <- matrix(NA,ncol=1,nrow=XG)
	logPY <- matrix(0,ncol=1,nrow=N)
	loglik <- 0

A <- t(matrix(rnorm(K*Q),nrow=Q,ncol=K))
theta <- matrix(rnorm(N*Q),ncol=Q,nrow=N)

muP <- matrix(0,N,Q) ## Matrix storing XB for each person and factor Q. 
SigmaP <- diag(Q)
acceptEAP <- matrix(0,K,1)
sigmamh <- matrix(.2,ncol=1,nrow=K)

for(ii in 1:XG){
	dum <- sampleZGP(theta,A,beta,Y,D,p1,p2,RR)
	Z <- dum$Z
	G <- dum$G
	logPYR <- dum$logPYR
	logPYD <- dum$logPYD	

	logPY[which(RR[,1]==1)] <- logPYR
	logPY[which(RR[,1]==0)] <- logPYD
    loglik <- ((ii - 1)*loglik + logPY)/ii

	theta <- SampleTheta(Z,A,muP,SigmaP)
  	## standardize theta for identification
  	theta <- standardize(theta)

	dum <- samplethreshold(beta,A,theta,sigmamh,G)
	beta <- dum$beta
	accept <- dum$accept
    acceptEAP <- ((ii - 1)*acceptEAP + accept)/ii
	if(ii%%10 == 0) sigmamh <- adjustMH(acceptEAP,sigmamh)

	A <- sampleloading(Z,theta,A,IC)
	#restrict A (variance and rotation)
	#A[1:Q,1:Q] <- 0
	#for(qq in 1:Q){
	#	A[qq,qq] <- 1
	#}

	##CAPS AEQ Data
	for(qq in 1:Q){
			if(qq==1){		#Socio-emotional scale
				A[1,-1] <- 0
				##A[1,1] <- abs(A[1,1])
			}
			if(qq==2){		##Expectency alcohol scale
				A[14,-2] <- 0
				##A[14,2] <- abs(A[14,2])
			}
			if(qq==3){		#Community scale
				A[5,-3] <- 0
				##A[5,3] <- abs(A[5,3])
				
			}
	}

## With regression:
  if(P>1){
	dum <- RegTheta(theta,X,SigmaP)
  	muP <- dum$muP
  	SigmaP <- dum$SigmaP
  	BReg <- dum$B
  }
  if(P == 1){
    muP <- rep(0,Q)
    Breg <- rep(0,Q)
    SigmaP0 <- diag(Q)
    SS <- crossprod(theta) + SigmaP0
    SigmaP <- rwishart(Q + N, chol2inv(chol(SS)))$IW  
  }


#storage
#	Mutheta[ii,] <-muP
	MBReg[ii,,] <- BReg
	MA[ii,] <- t(matrix(A,ncol=1,nrow=Q*K))
	MSigmaP[ii,] <- SigmaP[1:Q,1:Q]
	Mbeta[ii,] <- beta[,(2:C)]
	Mloglik[ii] <- sum(logPY)

	#EAP estimates
	if(ii > burnin){	
		iis <- (ii-(burnin+1)) + 1
		EAPtheta <- (theta + (iis-1)*EAPtheta)/iis
	}
}
	return(list(Mbeta=Mbeta,MSigmaP=MSigmaP,Mutheta=Mutheta,EAPtheta=EAPtheta,MA=MA,MBReg=MBReg,Mloglik=Mloglik,loglik=loglik))

}





sampleZGP <- function(theta,A,beta,Y,D,p1,p2,RR){

N <- nrow(Y)
K <- ncol(Y) 
C <- max(Y[!is.na(Y)])
Q <- ncol(theta)
G <- Z <- ZN <- matrix(0,N,K)
logPYR <- matrix(0,ncol=1,nrow=sum(RR[,1]==1))
logPYD <- matrix(0,ncol=1,nrow=sum(RR[,1]==0))

pkc <- array(0,c(N,K,C))
for(cc in 1:C){
      pkc[,,cc] = (pnorm(theta%*%t(A) - t(matrix(beta[,cc],ncol=N,nrow=K)))) - 
			(pnorm(theta%*%t(A) - t(matrix(beta[,cc+1],ncol=N,nrow=K)))) 
	}

for(kk in 1:K){
	low1 = theta%*%t(A)[,kk] - beta[kk,Y[,kk]+1]
	upp1 = theta%*%t(A)[,kk] - beta[kk,Y[,kk]]
	PY  = p1*(pnorm(upp1) - pnorm(low1)) + matrix((1.-p1)*p2,ncol=1,nrow=N)		## RR Probability
	PYN  = pnorm(upp1) - pnorm(low1)											## DQ Probability
  
	pdum <- matrix(0,ncol=C,nrow=N)
	for(cc in 1:C){
		group <- which(Y[,kk]==cc)
		groupn <- which(Y[,kk] != cc)
		pdum[group,cc] <- pkc[group,kk,cc]*(p1 + (1.-p1)*p2)/PY[group] 
		pdum[groupn,cc] <- pkc[groupn,kk,cc]*(1.-p1)*p2/PY[groupn]
	}
	for(ii in 1:N){
		G[ii,kk] <- sample(1:C, 1, prob = pdum[ii,1:C])
	}
		
	## loglik evaluation for DIC
	low2 = theta%*%t(A)[,kk] - beta[kk,G[,kk]+1]
	upp2 = theta%*%t(A)[,kk] - beta[kk,G[,kk]]
	low2N = theta%*%t(A)[,kk] - beta[kk,Y[,kk]+1]
	upp2N = theta%*%t(A)[,kk] - beta[kk,Y[,kk]]

	dummy <- log((pnorm(upp2) - pnorm(low2)))[which(RR[,kk]==1)]
	dummy[which(is.infinite(dummy))] <- 0
	logPYR <- logPYR + dummy
	dummy <- log((pnorm(upp2N) - pnorm(low2N)))[which(RR[,kk]==0)]
	dummy[which(is.infinite(dummy))] <- 0
	logPYD <- logPYD + dummy

#Sample Z RR data	
	u1 <- matrix(0,ncol=1,nrow=N) #RR 
	u2 <- matrix(0,ncol=1,nrow=N) #RR

	uppG <- which(G[,kk] == C)	
	lowG <- which(G[,kk] == 1)	
	midG <- which((G[,kk] != 1) & (G[,kk] != C))	
	u1[uppG] <-  pnorm(beta[kk,G[uppG,kk]] - (theta%*%t(A))[uppG,kk])
	u2[uppG] <- 1
	u1[lowG] <- 0
	u2[lowG] <- pnorm(beta[kk,G[lowG,kk]+1] - (theta%*%t(A))[lowG,kk]) 
	u1[midG] <-  pnorm(beta[kk,G[midG,kk]] - (theta%*%t(A))[midG,kk]) 
	u2[midG] <-  pnorm(beta[kk,G[midG,kk]+1] - (theta%*%t(A))[midG,kk]) 
	u <- u1 + runif(N)*(u2-u1)
	u[which(u > 1.-1e-07)] <- 1.-1e-07
	u[which(u < 1e-07)] <- 1e-07
      Z[which(D[,kk]==1),kk] = (theta%*%t(A))[which(D[,kk]==1),kk] + qnorm(u[which(D[,kk]==1)])
      Z[which(D[,kk]==0),kk] = (theta%*%t(A))[which(D[,kk]==0),kk] + rnorm(u[which(D[,kk]==0)])

#Sample Z DQ data
	u1 <- matrix(0,ncol=1,nrow=N) #DQ
	u2 <- matrix(0,ncol=1,nrow=N) #DQ

	uppY <- which(Y[,kk] == C)	
	lowY <- which(Y[,kk] == 1)	
	midY <- which((Y[,kk] != 1) & (Y[,kk] != C))	
	u1[uppY] <- pnorm(beta[kk,Y[uppY,kk]] - (theta%*%t(A))[uppY,kk])
	u2[uppY] <- 1
	u1[lowY] <- 0
	u2[lowY] <- pnorm(beta[kk,Y[lowY,kk]+1] - (theta%*%t(A))[lowY,kk]) 
	u1[midY] <- pnorm(beta[kk,Y[midY,kk]] - (theta%*%t(A))[midY,kk]) 
	u2[midY] <- pnorm(beta[kk,Y[midY,kk]+1] - (theta%*%t(A))[midY,kk]) 
	u <- u1 + runif(N)*(u2-u1)
	u[which(u > 1.-1e-07)] <- 1.-1e-07
	u[which(u < 1e-07)] <- 1e-07
      ZN[which(D[,kk]==1),kk] = (theta%*%t(A))[which(D[,kk]==1),kk] + qnorm(u[which(D[,kk]==1)])
      ZN[which(D[,kk]==0),kk] = (theta%*%t(A))[which(D[,kk]==0),kk] + rnorm(u[which(D[,kk]==0)])

	Z[which(RR[,kk]==0),kk] <- ZN[which(RR[,kk]==0),kk] #Define Z matrix of DQ and RR latent responses
	G[which(RR[,kk]==0),kk] <- Y[which(RR[,kk]==0),kk] 
}
	return(list(Z=Z,G=G,logPYR=logPYR,logPYD=logPYD))

}
	


samplethreshold <- function(beta,A,theta,sigmamh,G){

  N <- nrow(G)	
  K <- nrow(beta)
  accept <- rep(0,K)
  ##init betaster (voor lower and upper tresholds)
  betaster <- beta
  C <- max(G[!is.na(G)])
  
for(kk in 1:K){ 
  
  ## generate proposal
	for(cc in 2:C){
			low	<- (betaster[kk,cc-1]-beta[kk,cc])/sigmamh[kk]
			upp	<- (beta[kk,cc+1]-beta[kk,cc])/sigmamh[kk]
			U1	<- pnorm(low)
			U2	<- pnorm(upp)
			UU  <- U1 + runif(1)*(U2-U1)
		      if(UU > (1.-1e-07)) UU <- 1.-1e-07
      	      if(UU < 1e-07) UU <- 1e-07
			betaster[kk,cc] <- beta[kk,cc] + sigmamh[kk]*qnorm(UU)
	}
	temp1 = 1
	temp2 = 1
	## prior
      for(dd in 2:C){							
            NUM = pnorm((beta[kk,dd+1] - beta[kk,dd])/sigmamh[kk]) - 
			pnorm((betaster[kk,dd-1] - beta[kk,dd])/sigmamh[kk])
            DEN =	pnorm((betaster[kk,dd+1] - betaster[kk,dd])/sigmamh[kk]) -  
                	pnorm((beta[kk,dd-1] - betaster[kk,dd])/sigmamh[kk])
		temp1 = temp1 * (NUM/DEN)
	}

## likelihood
        ee = G[,kk] + 1	  
		NUM = pnorm((theta%*%t(A))[,kk] - matrix(betaster[kk,ee-1],ncol=1,nrow=N)) - 
						pnorm((theta%*%t(A))[,kk] - matrix(betaster[kk,ee],ncol=1,nrow=N))
	      DEN = pnorm((theta%*%t(A))[,kk] - matrix(beta[kk,ee-1],ncol=1,nrow=N)) - 
						pnorm((theta%*%t(A))[,kk] - matrix(beta[kk,ee],ncol=1,nrow=N))
      	temp2 = temp2 * prod(NUM/DEN) 

  ## Evaluate with uniform probability. 
		R = temp1 * temp2
    if (runif(1) < R){
      beta[kk,] <- betaster[kk,]
      accept[kk] <- 1
    }
  
 } ## end loop over kk = 1,K		
  
return(list(beta=beta, accept=accept))
} ## end function

SampleTheta2 <- function(Z,A,muP,SigmaP){
  # prior theta: theta ~ MVN(muP,SigmaP) = MVN(0,I)
  # also possible to define as: t(Z) = A t(theta) + E, E ~ MVN(0,I) 
  N <- nrow(Z)
  K <- ncol(Z)
  theta <- matrix(0,N,2)

  for(ii in 1:N){
    theta[ii,] <- SampleB(matrix(Z[ii,],ncol=1),A,1,muP[ii,],SigmaP)$B
  }
    return(theta)
}

RegTheta <- function(theta,X,SigmaP){
  ## X =  matrix with covariates to regress on theta
  ## first colum =  1s for the intercept
  ## prior on the intercept: N(0,.00001) for identification -> rescale to 0 after sampling step
  ## 
  Q <- ncol(theta)
  N <- nrow(theta)
  P <- ncol(X)
  mu0 <- rep(0,P*Q)
  Sigma0 <- diag(P*Q)*3 ## slightly-informative prior
  Sigma0[1,1] <- Sigma0[(P+1),(P+1)] <- .00001 ## very informative to fix intercept at 0. 
  dum <- SampleB(theta,X,SigmaP,mu0,Sigma0)
  B <- dum$B
  B <- matrix(B,ncol=Q)
  B[1,] <- 0
  muP <- X %*% B
  
  SigmaP0 <- diag(Q)
  SS <- crossprod(theta - muP) + SigmaP0
  SigmaP <- rwishart(Q + N, chol2inv(chol(SS)))$IW
  
  return(list(B=B,muP=muP,SigmaP=SigmaP))
}


SampleTheta <- function(Z,A,muP,SigmaP){
#routine JP
N <- nrow(Z)
K <- ncol(Z)
Q <- ncol(A)
		
		Sigmatinv <- solve(SigmaP)
		covar <- solve(t(A)%*%A + Sigmatinv)
		thetahat <- covar%*%(t(A)%*%t(Z) + t(muP%*%Sigmatinv))
		cholcovar <- chol((covar+t(covar))/2)	#force symmetric
		theta	 <- cholcovar %*% matrix(matrix(mvrnorm(N,mu=rep(0,Q),Sigma=diag(Q)),ncol=1),ncol=N,nrow=Q) + thetahat
		theta <- t(matrix(theta,ncol=N,nrow=Q))

return(theta)

}

SampleB <- function(Y,X,Sigma,B0,V0){
  ## multivariate normal regression of Y(N,K) on X(N,p)
  ## returns: regression coefficients (as vector), multivariate predictor
  ## model: Y = XB + E, E(0,Sigma)
  ## prior: vec(B) ~ N(vec(B0),V0)
  ## author: Rinke Klein Entink, adapted from Peter Rossi's bayesm package
  ## kroneckerproduct %x%

  m <- ncol(X)
  k <- ncol(Y)
  V0 <- solve(V0)
  Bvar <- solve(solve(Sigma %x%solve(crossprod(X)))+V0)
  Btilde <- Bvar%*%((solve(Sigma)%x%diag(m))%*%matrix(crossprod(X,Y),ncol=1)+ V0%*%matrix(B0,ncol=1))
  B <- Btilde + chol(Bvar)%*% matrix(rnorm(length(Btilde)),ncol=1)
  pred <- X %*% matrix(B,ncol=k)
  return(list(B=B,pred=pred))
}


rwishart <- function (nu, V) 
{
  ## adopted from bayesm package, written by Peter Rossi.
  ## returns a draw from an inverse-Wishart distribution.
  m = nrow(V)
  df = (nu + nu - m + 1) - (nu - m + 1):nu
  if (m > 1) {
    T = diag(sqrt(rchisq(c(rep(1, m)), df)))
    T[lower.tri(T)] = rnorm((m * (m + 1)/2 - m))
  }
  else {
    T = sqrt(rchisq(1, df))
  }
  U = chol(V)
  C = t(T) %*% U
  CI = backsolve(C, diag(m))
  return(list(IW = crossprod(t(CI))))
}

sampleloading <- function(Z,theta,A,IC){

#item specific factor-loadings 
	N <- nrow(Z)
	K <- ncol(Z)

	if(IC){
		Q <- ncol(theta)-1
		SigmaA0inv <- diag(Q)
		A0 <- rep(0,Q)
		intercept <- theta[,1]
	}
	else{
		Q <- ncol(theta)
		#prior factor loadings require better specification
		SigmaA0inv <- diag(Q)
		A0 <- rep(0,Q)
		intercept <- matrix(0,N,1)
	}
	
##	Per Item k  ## adapt dimension Q with or without intercept
	for(kk in 1:K){
		if(IC){
			dumX <- theta[,2:(Q+1)] 			
		}
		else{
			dumX <- theta[,1:Q] 			
		}
		Ahat <- t(dumX) %*% Z[,kk]
		covar <- solve(t(dumX)%*%dumX + SigmaA0inv)
		muA <- covar %*% Ahat + SigmaA0inv%*%A0
		if(IC){
			A[kk,2:(Q+1)] <- mvrnorm(1,mu=t(muA),Sigma=covar)
		}
		else{
			A[kk,1:Q] <- mvrnorm(1,mu=t(muA),Sigma=covar)
		}	
	}
	return(A)
}

sampleP <- function(theta,muP,SigmaP){

	N <- nrow(theta) 
	Q <- ncol(theta)
	SigmaP0 <- diag(Q)
	muP0 <- rep(0,Q)

	SS <- crossprod(theta - muP) + SigmaP0
	SigmaP <- rwishart(Q + N, chol2inv(chol(SS)))$IW

	#Sigmahat <- solve(solve(SigmaP/N) + diag(2))
	#muhat <- Sigmahat%*%(matrix((apply(theta,2,mean) + muP0),2,1))
	#muP <- mvrnorm(1,mu=t(muhat),Sigma=Sigmahat)

	return(list(muP=muP,SigmaP=SigmaP))
}


adjustMH <- function(refresh,sigmamh){

K <- length(sigmamh)

for(kk in 1:K){
	if(refresh[kk] > .6){
		sigmamh[kk] = sigmamh[kk] + .01
	}	
	if(refresh[kk] < .2){
		if(sigmamh[kk] > .02){
			sigmamh[kk] = sigmamh[kk] - .01
		}
	}
}

return(sigmamh)

}


standardize <- function (X) 
{
  if (is.vector(X)) {
    X <- (X - mean(X))/sd(X)
  }
  if (is.matrix(X)) {
    X <- X - t(matrix(apply(X, 2, mean), nrow = ncol(X), 
                      ncol = nrow(X)))
    sds <- t(matrix(1/apply(X, 2, sd), nrow = ncol(X), ncol = nrow(X)))
    X <- X * sds
  }
  return(X)
}